#include "libuvc/libuvc.h"
#include <stdio.h>
#include <opencv/cv.h>
#include <opencv/highgui.h>

#include "ipevo_cam.h"

void cb(uvc_frame_t *frame, void *ptr) {
	//puts("cb! cb!");

	uvc_frame_t *bgr;
	uvc_error_t ret;

	/* We'll convert the image from YUV/JPEG to BGR, so allocate space */
	bgr = uvc_allocate_frame(frame->width * frame->height * 3);
	if (!bgr) {
		puts("unable to allocate bgr frame!");
		return;
	}


	ret = uvc_mjpeg2rgb(frame, bgr);
	if (ret) {
		puts("unable to uvc_mjpeg2rgb!");
		uvc_perror(ret, "uvc_mjpeg2rgb");
		uvc_free_frame(bgr);
		return;
	}

	//Use opencv.highgui to display the image:

	IplImage* cvImg = cvCreateImageHeader(
		cvSize(bgr->width, bgr->height),
		IPL_DEPTH_8U,
		3);

	cvSetData(cvImg, bgr->data, bgr->width * 3);

	cvConvertImage(cvImg, cvImg, CV_CVTIMG_SWAP_RB);			//convert RGB to BGR

	//cvNamedWindow("Test", CV_WINDOW_AUTOSIZE);
	cvShowImage("Test", cvImg);
	cvWaitKey(10);

	cvReleaseImageHeader(&cvImg);
	uvc_free_frame(bgr);
}

void print_status_callback(char status_type[], short status_value)
{
	printf("print_status_callback: %s : %d \n", status_type, status_value);
}

/////////////////////////////////////////
int main() {

	Ziggi_HD_Plus ziggiHDplus_instance;
	init_Ziggi_HD_Plus(&ziggiHDplus_instance);

	const uvc_format_desc_t *fmt_desc = ziggiHDplus_instance.getStreamFormat();
	
	///print frame-format info
	/*
	for (; fmt_desc; fmt_desc = fmt_desc->next)
	{
		uvc_frame_desc_t *frame_desc;
		int i;
		switch (fmt_desc->bDescriptorSubtype)
		{
		case UVC_VS_FORMAT_UNCOMPRESSED:
		case UVC_VS_FORMAT_MJPEG:
		case UVC_VS_FORMAT_FRAME_BASED:

			printf(
				"\t\%s(%d)\n"
				"\t\t  bits per pixel: %d\n"
				"\t\t  GUID: ",
				"",// _uvc_name_for_format_subtype(fmt_desc->bDescriptorSubtype),
				fmt_desc->bFormatIndex,
				fmt_desc->bBitsPerPixel);

			for (i = 0; i < 16; ++i)
				printf("%02x", fmt_desc->guidFormat[i]);

			printf(" (%4s)\n", fmt_desc->fourccFormat);

			printf(
				"\t\t  default frame: %d\n"
				"\t\t  aspect ratio: %dx%d\n"
				"\t\t  interlace flags: %02x\n"
				"\t\t  copy protect: %02x\n",
				fmt_desc->bDefaultFrameIndex,
				fmt_desc->bAspectRatioX,
				fmt_desc->bAspectRatioY,
				fmt_desc->bmInterlaceFlags,
				fmt_desc->bCopyProtect);


			//DL_FOREACH(fmt_desc->frame_descs, frame_desc)
			for (frame_desc = fmt_desc->frame_descs; frame_desc; frame_desc = frame_desc->next)
			{
				uint32_t *interval_ptr;

				printf(
					"\t\t\tFrameDescriptor(%d)\n"
					"\t\t\t  capabilities: %02x\n"
					"\t\t\t  size: %dx%d\n"
					"\t\t\t  bit rate: %d-%d\n"
					"\t\t\t  max frame size: %d\n"
					"\t\t\t  default interval: 1/%d\n",
					frame_desc->bFrameIndex,
					frame_desc->bmCapabilities,
					frame_desc->wWidth,
					frame_desc->wHeight,
					frame_desc->dwMinBitRate,
					frame_desc->dwMaxBitRate,
					frame_desc->dwMaxVideoFrameBufferSize,
					10000000 / frame_desc->dwDefaultFrameInterval);

				if (frame_desc->intervals) {
					for (interval_ptr = frame_desc->intervals;
						*interval_ptr;
						++interval_ptr) {
						printf(
							"\t\t\t  interval[%d]: 1/%d\n",
							(int)(interval_ptr - frame_desc->intervals),
							10000000 / *interval_ptr);
					}
				}
				else {
					printf(
						"\t\t\t  min interval[%d] = 1/%d\n"
						"\t\t\t  max interval[%d] = 1/%d\n",
						frame_desc->dwMinFrameInterval,
						10000000 / frame_desc->dwMinFrameInterval,
						frame_desc->dwMaxFrameInterval,
						10000000 / frame_desc->dwMaxFrameInterval);
					if (frame_desc->dwFrameIntervalStep)
						printf(
						"\t\t\t  interval step[%d] = 1/%d\n",
						frame_desc->dwFrameIntervalStep,
						10000000 / frame_desc->dwFrameIntervalStep);
				}
			}

			break;
		default:

			printf("\t-UnknownFormat (%d)\n",
				fmt_desc->bDescriptorSubtype);
		}
	}
	///	end print frame info
	*/

	//UVC_FRAME_FORMAT_YUYV
	//UVC_FRAME_FORMAT_RGB		
	//UVC_FRAME_FORMAT_MJPEG
	ziggiHDplus_instance.setStreamFormat(UVC_FRAME_FORMAT_MJPEG, 1024, 768, 30);
	ziggiHDplus_instance.setStatusCallback(print_status_callback);

	ziggiHDplus_instance.startStreamming(cb);
	ziggiHDplus_instance.startFocus();	

	int c;
	do {
		if (c != '\r' &&  c!= '\n')
		{
			printf("\n");
			printf("IPEVO CAM DEMO \n");
			printf("------------------------------- \n");
			printf("f => focus \n");
			printf("1 => set Brightness value to -32 \n");
			printf("2 => set Brightness value to 0 \n");
			printf("3 => set Brightness value to 32 \n");
			printf("9 => get Exposure value \n");
			printf("0 => get Brightness/White Balance Maximum, Minimum, Current and Default value \n");
			printf("m => stop streamming \n");
			printf("q => quit this application \n");
			printf("\n");
			fflush(stdout);
		}
		
		c = getchar();						
		
		if (c == 'f')
		{
			ziggiHDplus_instance.startFocus();
		}
		else if (c == '1')
		{
			ziggiHDplus_instance.setBrightness(-32);
		}
		else if (c == '2')
		{
			ziggiHDplus_instance.setBrightness(0);
		}
		else if (c == '3')
		{
			ziggiHDplus_instance.setBrightness(32);
		}
		else if (c == '0')
		{
			short default_brightness = 0, max_brightness = 0, min_brightness = 0, curr_brightness = 0;
			UVC_PropertyValueType propertyType = Default;
			ziggiHDplus_instance.getBrightness(&default_brightness, propertyType);		//0
			propertyType = Maximum;
			ziggiHDplus_instance.getBrightness(&max_brightness, propertyType);			//64
			propertyType = Minimum;
			ziggiHDplus_instance.getBrightness(&min_brightness, propertyType);			//-64
			propertyType = Current;
			ziggiHDplus_instance.getBrightness(&curr_brightness, propertyType);
			printf("brightness: %d %d %d %d \n", default_brightness, max_brightness, min_brightness, curr_brightness);

			//testing
			short default_uvc_property = 0, max_uvc_property = 0, min_uvc_property = 0, curr_uvc_property = 0;

			ziggiHDplus_instance.setWhiteBalance(4600);
			propertyType = Default;
			ziggiHDplus_instance.getWhiteBalance(&default_uvc_property, propertyType);
			propertyType = Maximum;
			ziggiHDplus_instance.getWhiteBalance(&max_uvc_property, propertyType);
			propertyType = Minimum;
			ziggiHDplus_instance.getWhiteBalance(&min_uvc_property, propertyType);
			propertyType = Current;
			ziggiHDplus_instance.getWhiteBalance(&curr_uvc_property, propertyType);
			printf("uvc property: %d %d %d %d \n", default_uvc_property, max_uvc_property, min_uvc_property, curr_uvc_property);
		}
		else if (c == '9')
		{
			//ziggiHDplus_instance.setExposure(10);
			short evv;
			UVC_PropertyValueType propertyType = Current;
			ziggiHDplus_instance.getExposure(&evv, propertyType);
			printf("ev %d\n", evv);

			/*
			char firmwareVer[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
			ziggiHDplus_instance.getFirmwareVersion(firmwareVer);
			int kk = 0;
			for (kk = 0; kk < 8; kk++)
			{
				printf(" %2x ", firmwareVer[kk]);
			}
			printf("\n");
			*/
		}
		else if (c == 'm')
		{
			ziggiHDplus_instance.stopStreamming();
		}
	} while (c != 'q');

	
	
}

